var fs = require("fs");
var gpath = require("path");
var crypto = require('crypto');
var colors = require("colors");
var needle = require('needle');
var minimatch = require("minimatch");

var _config, USERKEY;

var fileList = [];
var host = "http://upload.ws.netease.com",
	port = 80;
var log = {
	alert: function(text) {
		console.log(text.yellow);
	},
	err: function(text) {
		console.log(text.red);
	},
	success: function(text) {
		console.log(text.green);
	}
}

function checkExclude(path) {
	var flag = true;
	_config.exclude && _config.exclude.map(function(_p) {
		if (minimatch(gpath.join("/", path), gpath.join("/", _p))) {
			flag = false
		}
	})
	return flag;
}

function ftp(config) {
	var successArr = [];
	var failArr = [];
	return new Promise(function(reslove, reject) {
		var _files = {
				userKey: USERKEY.data,
				list: []
			},
			$p;
		_config = config
		if (config.file) {
			// 上传文件
			log.alert("正在上传" + config.file);
			$p = new Promise(function(resolve, reject) {
				_files.list.push(uploadPath(config.file, gpath.basename(config.file)));
				resolve(_files);
			})
		} else if (config.files) {
			// 上传文件夹
			log.alert("正在读取" + config.files + "下的文件");
			$p = new Promise(function(resolve, reject) {
				readFiles(config.files, function(path) {

					if (checkExclude(gpath.join(path[1], path[2]))) {
						_files.list.push(path);
					}
				}, function() {
					resolve(_files);
				}, function() {
					reject(config.files + "目录不存在");
				});
			})
		}
		return $p.then(function(files) {
			return Promise.all(files.list.map(function(file) {
				return getUploadData(files.userKey, file).then(function(uploadData) {
					return upload(uploadData)
				}).then(function(success) {
					successArr.push({
						error: null,
						url: success.url
					})
					log.success("上传" + success.url + "成功");
				}).catch(function(err) {
					failArr.push({
						error: err,
						url: err.url
					})
					log.err("上传" + err.url + "失败，原因：" + JSON.stringify(err));
				})
			}))
		}).then(function(values) {
			log.success("完成！！！！！！！！！！");
			reslove([successArr, failArr]);
		})
	})
}

function verifyUser(config) {
	return new Promise(function(resolve, reject) {
		log.alert("正在认证用户");
		needle.request('post', host + "/fileupload/auth", {
			username: config.username,
			password: config.password
		}, {
			json: true
		}, function(err, resp, body) {
			log.success("认证用户成功");
			if (body.resultcode == "8200") {
				USERKEY = body
				resolve();
			} else {
				reject(body);
			}
		});
	})
}

function uploadPath(path, filename) {

	var parentUrl = gpath.resolve(path, ".."),
		onlinePath;
	if (_config.file) {
		onlinePath = gpath.join(_config.online)
	} else if (_config.files) {
		onlinePath = gpath.join(_config.online, parentUrl.replace(_config.files, ""))
	}
	return [path, parentUrl.replace(_config.files, ""), filename, onlinePath.replace(/\\/g, "/")];
}

function readFiles(rootpath, callback, complete, error) {
	function walk(rootpath, callback, error) {
		var files = fs.readdirSync(rootpath);
		if (!files) {
			error && error();
			return;
		}
		files.forEach(function(filename, index) {
			var stats = fs.statSync(gpath.join(rootpath, filename))
			if (stats.isFile()) {
				callback && callback(uploadPath(gpath.join(rootpath, filename), filename), stats);
			} else if (stats.isDirectory()) {
				walk(gpath.join(rootpath, filename), callback, complete);
			}
		})
	}
	walk(rootpath, function(path, stats) {
		callback && callback(path, stats)
	}, error)
	complete && complete();
}

function getContentType(ext) {
	var contentType = [{
			"text/html": [".html"]
		}, {
			"text/plain": []
		}, {
			"text/xml": [".svg"]
		}, {
			"text/css": [".css", ".less", ".sass"]
		}, {
			"image/gif ": [".gif"]
		}, {
			"image/jpeg": [".jpeg", ".jpg"]
		}, {
			"image/png": [".png"]
		}, {
			"application/x-javascript": [".js", ".map", ".vue"]
		}, {
			"application/vnd.android.package-archive": [".apk"]
		}, {
			"audio/mp3": [".mp3"]
		}, {
			"video/mpeg4": [".mp4"]
		}, {
			"application/font-woff": [".woff"]
		}, {
			"application/x-font-ttf": [".ttf"]
		}, {
			"video/x-flv": [".flv"]
		}],
		r = "";
	contentType.forEach(function(_d) {
		for (var i in _d) {
			if (_d[i].indexOf(ext) !== -1) {
				r = i
			}
		}
	});
	return r;
}

function getUploadData(key, path) {
	return new Promise(function(resolve, reject) {
		var stream = fs.createReadStream(path[0]);
		var fsHash = crypto.createHash('md5');
		var buffer = fs.readFileSync(path[0]);
		stream.on('data', function(d) {
			fsHash.update(d);
		});
		stream.on('end', function() {
			resolve({
				file: {
					file: path[0],
					// buffer: buffer,
					content_type: getContentType(gpath.extname(path[0]))
				},
				filename: path[2],
				md5: fsHash.digest('hex'),
				session: key,
				path: path[3]
			})
		});
		stream.on("error", function() {
			reject("读取文件" + path[0] + "失败");
		})
	})
}

function upload(formData) {
	return new Promise(function(resolve, reject) {
		needle.post(host + "/fileupload/add", formData, {
			multipart: true
		}, function(err, resp, body) {
			if (body) {
				body.url = formData.file.file;
				if (body.resultcode == "200") {
					resolve(body);
				} else {
					reject(body);
				}
			} else {
				reject({
					url: formData.file.file,
					msg: "未知错误"
				});
			}
		});
	})
}
module.exports = {
	verify: verifyUser,
	upload: ftp
};

